unit fFunc00004;

interface

uses fFunc, fUtils, OpenGL, GL;

type
	TfrmFunc00004 = class(TfrmFunc)
  public
    procedure glInit; override;
    procedure glDraw; override;
    procedure glResize; override;
  end;

const
  // Material Constants
  mat_ambient : array [0..3] of GLfloat = ( 0.5, 0.5, 0.0, 0.15 );
  mat_specular : array [0..3] of GLfloat = (  1.0, 1.0, 1.0, 0.15 );
  mat_shininess : GLfloat = 15.0;
  mat_torus : array [0..3] of GLfloat = ( 0.75, 0.75, 0.0, 1.0 );
  mat_cylinder : array [0..3] of GLfloat = ( 0.0, 0.75, 0.75, 0.15 );

  // Light Constants
  position : array [0..3] of GLfloat = ( 0.0, 0.0, 1.0, 1.0 );

var
  // User Variables
  spin : GLfloat = 0.0;              	// Var used for the amount to rotate an object by
  EyePosition : GLboolean = GL_FALSE;
  // Quadric for our Cylinder
  CylinderQuadratic : gluQuadricObj;
  TorusDL : GLuint; 									// Variable For our torus Display list

implementation

{ TfrmFunc00004 }

procedure TfrmFunc00004.glDraw;
begin
  glClearColor(0.0, 0.0, 0.0, 1.0);     // Black Background
  glClear (GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);        // Clear the colour buffer
  glLoadIdentity();                     // Load a "clean" model matrix on the stack
  glTranslatef(0.0,0.0,-10.0);          // Move the scene back 10 units so we can see the triangle properly


//  glLightfv(GL_LIGHT0, GL_POSITION, @position);          // Set the position of the light


  SpinDisplay(spin, 2.0);                                // Get the next Rotation Angle for the scene
  glRotatef(spin,1.0,1.0,1.0);                           // Rotate the scene on all axes
  glPushMatrix();                                        // Copy the current matrix and push it onto the stack
    glPushMatrix();                                      // Copy the current matrix and push it onto the stack
        glMaterialfv(GL_FRONT, GL_DIFFUSE,@mat_torus);   // Set the Material Properties for the Torus
        glCallList(TorusDL);                             // Render the Torus to the scene
    glPopMatrix();                                       // Restore the previous Matrix

    glEnable(GL_BLEND);                                  // Enable Belnding
    glDepthMask(GL_FALSE);                               // Lock the Depth Mask so we cant edit it
    glBlendFunc(GL_SRC_ALPHA, GL_ONE);                   // Set the type of blending we want
    glMaterialfv(GL_FRONT, GL_DIFFUSE, @mat_cylinder);   // Set the Material properties for the cylinder
    gluCylinder(CylinderQuadratic,0.5,0.5,3.5,32,32);	   // A Cylinder With A Radius Of 0.5 And A Height Of 2
    glDepthMask(GL_TRUE);                                // Unlock the Depth Mask so we can edit it again
    glDisable(GL_BLEND);                                 // Disable Blending
  glPopMatrix();                                         // Restore the previously saved Matrix


  // Flush the OpenGL Buffer
  glFlush();                            // ( Force the buffer to draw or send a network packet of commands in a networked system)

	FContext.PageFlip;
  
end;

procedure TfrmFunc00004.glInit;
begin
  glClearColor(0.0, 0.0, 0.0, 1.0); // Black Background
  glShadeModel(GL_SMOOTH);          // Use Smooth shading ( This is the Default so we dont actually have to set it) 

  // Generate a sphere using the OpenGL utility Library
  CylinderQuadratic := gluNewQuadric();		       // Create A Pointer To The Quadric Object (Return 0 If No Memory) (NEW)
  gluQuadricNormals(CylinderQuadratic, GLU_SMOOTH);	       // Create Smooth Normals (NEW)

  // Set the Material properties
  glMaterialfv(GL_FRONT, GL_AMBIENT, @mat_ambient);
  glMaterialfv(GL_FRONT, GL_SPECULAR, @mat_specular);
  glMaterialfv(GL_FRONT, GL_SHININESS, @mat_shininess);

  glEnable(GL_LIGHTING);             // Enable Lighting
  glEnable(GL_LIGHT0);               // Enable Light 0
  glDepthFunc(GL_LEQUAL);            // Set the Depth Function to Linear Equal
  glEnable(GL_DEPTH_TEST);           // Enable Depth Testing

  CreateTorus(TorusDL, 0.5, 1.4, 128, 256); // Create a torus in a display list

end;

procedure TfrmFunc00004.glResize;
begin
	with FP do
  begin
    if (Height = 0) then                // prevent divide by zero exception
      Height := 1;
    glViewport(0, 0, Width, Height);    // Set the viewport for the OpenGL window
    glMatrixMode(GL_PROJECTION);        // Change Matrix Mode to Projection
    glLoadIdentity();                   // Reset View
    gluPerspective(30.0, Width/Height, 1.0, 20.0);  // Do the perspective calculations. Last value = max clipping depth

    glMatrixMode(GL_MODELVIEW);         // Return to the modelview matrix
    glLoadIdentity();                   // Reset View
    glFlush();
  end;
end;

end.
 