unit fFunc00010;

interface

uses fFunc, OpenGL, GL;

type
	TfrmFunc00010 = class(TfrmFunc)
  protected
		procedure DrawPoints();
    procedure DrawLines();
    procedure DrawLineStrip();
    procedure DrawLineLoop();
    procedure DrawPolygon();
    procedure DrawQuad();
    procedure DrawQuadStrip();
    procedure DrawTriangles();
    procedure DrawTriangleStrip();
    procedure DrawTriangleFan();
  public
    procedure glInit; override;
    procedure glDraw; override;
    procedure glResize; override;
    procedure DO_TimerTimer(Sender: TObject); override;
  end;

const
  // User Constants
  DRAW_POINTS = 1;
  DRAW_LINES = 2;
  DRAW_LINE_STRIP = 3;
  DRAW_LINE_LOOP = 4;
  DRAW_POLYGON = 5;
  DRAW_QUADS = 6;
  DRAW_QUAD_STRIP = 7;
  DRAW_TRIANGLES = 8;
  DRAW_TRIANGLE_STRIP = 9;
  DRAW_TRIANGLE_FAN = 10;

  StateDescriptions : array[1..10] of string =
      ('Points',
      'Lines',
      'Line Strip',
      'Line Loop',
      'Polygon',
      'Quads',
      'Quad Strip',
      'Triangles',
      'Triangle Strip',
      'Triangle Fan');
var

  // User Variables

  spin : GLfloat = 0.0;              // Var used for the amount to rotate an object by

  CurrentDrawState : GLInt = 1;      // The current drawing sequence

implementation

{ TfrmFunc00010 }

procedure TfrmFunc00010.DO_TimerTimer(Sender: TObject);
begin
	CurrentDrawState := CurrentDrawState + 1;
  if CurrentDrawState > 10 then   // if the draw state is greater than the max number of states defined
  	CurrentDrawState := 1;        // set the state back to the first in the list
  inherited;
end;

procedure TfrmFunc00010.DrawLineLoop;
begin
  // Begin by specifying what Primitive we are going to draw
  // In this case GL_LINE_LOOP...
  // When drawing GL_LINE_LOOP you also do not have to worry about an even number of calls to glVertex*
  glBegin(GL_LINE_LOOP);
    glVertex2f(-1.0,1.0);                // Set the begin position of the line in the scene
    glVertex2f(1.0,0.5);                 // Set the end position of tthe previous line and the begin position of the next line in the scene
    glVertex2f(5.0,10.0);
    glVertex2f(3.0,-4.0);
    glVertex2f(-8.0,-5.0);
    glVertex2f(9.0,-5.0);
    glVertex2f(-19.0,-9.0);              // Set the end of the last line ( there will be an extra line drawn from this point to the first point specified
  glEnd;                                 // End the current set of rendering instructions
end;

procedure TfrmFunc00010.DrawLines;
begin
  // Begin by specifying what Primitive we are going to draw
  // In this case LINES...
  // When drawing GL_LINES make sure that you make an even number of calls to glVertex*
  // If the calls are not even numbered, the last call wil be discarded.
  glBegin(GL_LINES);
    glVertex2f(-1.0,1.0);                // Set the begin position of the line in the scene
    glVertex2f(1.0,0.5);                 // Set the end position of the line in the scene
    glVertex2f(5.0,10.0);
    glVertex2f(3.0,-4.0);
    glVertex2f(-8.0,-5.0);
    glVertex2f(9.0,-5.0);
  glEnd;                                 // End the current set of rendeing instructions
end;

procedure TfrmFunc00010.DrawLineStrip;
begin
  // Begin by specifying what Primitive we are going to draw
  // In this case GL_LINE_STRIP...
  // When drawing GL_LINE_STRIPS you do not have to worry about an even number of calls to glVertex*
  glBegin(GL_LINE_STRIP);
    glVertex2f(-1.0,1.0);                // Set the begin position of the line in the scene
    glVertex2f(1.0,0.5);                 // Set the end position of tthe previous line and the begin position of the next line in the scene
    glVertex2f(5.0,10.0);
    glVertex2f(3.0,-4.0);
    glVertex2f(-8.0,-5.0);
    glVertex2f(9.0,-5.0);
    glVertex2f(19.0,-9.0);               // Set the end of the last line
  glEnd;   
end;

procedure TfrmFunc00010.DrawPoints;
begin
   // Begin by specifying what Primitive we are going to draw
   // In this case Points...
  glBegin(GL_POINTS);
    glVertex2f(-1.0,1.0);                // Set a points position on screen
    glVertex2f(1.0,0.5);                 // ( these calls may not be made outside of a glBegin and glEnd block)
    glVertex2f(5.0,10.0);
    glVertex2f(3.0,-4.0);
    glVertex2f(-8.0,-5.0);
  glEnd;                                 // End the current set of rendeing instructions
end;

procedure TfrmFunc00010.DrawPolygon;
begin
  // Begin by specifying what Primitive we are going to draw
  // In this case GL_POLYGON...
  // A polygon is any non convex shape with three or more sides
  // Always draw the points of a polygon in a counter clockwise direction
  // For an explanation on this check out chapter 2 of the redbook under the heading "polygons"
  glBegin(GL_POLYGON);
    glVertex2f(-10.0,-10.0);               // Set the first point of the polygon in the scene (bottom left)
    glVertex2f( 10.0,-10.0);               // Set the second point in the scene (bottom right)
    glVertex2f( 10.0, 10.0);               // Top right
    glVertex2f( 0.0,  20.0);               // Some point at 0.0 on the x axis and 20 units up the y axis
    glVertex2f(-10.0, 10.0);               // Top Left
  glEnd();                                 // End the current set of rendering instructions
end;

procedure TfrmFunc00010.DrawQuad;
begin
  // Begin by specifying what Primitive we are going to draw
  // In this case GL_QUAD...
  // A Quad is a polygon with only four sides
  // Always draw the points of a Quad in a counter clockwise direction
  // For an explanation on this check out chapter 2 of the redbook under the heading "polygons"
  // NB: You can draw more than one QUAD between the glBegin and glEnd
  // But make sure that you have 4 points/vertices for each quad 
  glBegin(GL_QUADS);
    glVertex2f(-10.0,-10.0);               // Set the first point of the polygon in the scene (bottom left)
    glVertex2f( 10.0,-10.0);               // Set the second point in the scene (bottom right)
    glVertex2f( 10.0, 10.0);               // Top right
    glVertex2f(-10.0, 10.0);               // Top Left

    glVertex2f(-40.0,-30.0);               // Set the first point of the polygon in the scene (bottom left)
    glVertex2f(  0.0,-30.0);               // Set the second point in the scene (bottom right)
    glVertex2f(  0.0,-15.0);               // Top right
    glVertex2f(-40.0,-15.0);               // Top Left
  glEnd();                                 // End the current set of rendering instructions
end;

procedure TfrmFunc00010.DrawQuadStrip;
begin
  // Begin by specifying what Primitive we are going to draw
  // In this case GL_QUAD_STRIP...
  // A Quad is a polygon with only four sides
  // Always draw the points of a Quad in a counter clockwise direction
  // For an explanation on this check out chapter 2 of the redbook under the heading "polygons"
  glBegin(GL_QUAD_STRIP);
    glColor3f(1.0,0.0,0.0);
    glVertex2f(-10.0,-10.0);               // Bottom Left
    glVertex2f(-10.0, 10.0);               // Top Left
    glVertex2f( 10.0,-10.0);               // Bottom Right
    glVertex2f( 10.0, 10.0);               // Top Right
    glColor3f(0.0,1.0,0.0);
    glVertex2f( 20.0,-10.0);               // Bottom right
    glVertex2f( 20.0, 10.0);               // Top right
    glColor3f(0.0,0.0,1.0);
    glVertex2f( 30.0,-12.0);               // Bottom right
    glVertex2f( 30.0, 12.0);              // Top right
  glEnd();                                 // End the current set of rendering instructions
end;

procedure TfrmFunc00010.DrawTriangleFan;
var CirclePoints, i : GLint;
    Angle : GLfloat;
begin
  // Begin by specifying what Primitive we are going to draw
  // In this case GL_TRIANGLE_FAN...
  // A TRIANGLE_FAN is a series of trinagles that are positioned so that the all share one middle vertex
  
  CirclePoints := 10;                          // Number of point to use for the tiangle fan

  // Recursion is not always a good idea to use when rendering a sequence of frames (animation).
  // Rather use a Display list ( will be covered in Chapter 4 of the Redbook) 

  glBegin(GL_TRIANGLE_FAN);
    for i := 0 to CirclePoints -1 do
    begin
      Angle := 2*PI*i/CirclePoints;
      glVertex2f(cos(angle)*20, sin(angle)*20); // Draw the next vertex in a circle
    end;
  glEnd();                                      // End the current set of rendering instructions
end;

procedure TfrmFunc00010.DrawTriangles;
begin
  // Begin by specifying what Primitive we are going to draw
  // In this case GL_TRIANGLES...
  // A Triangle is a polygon with only three sides
  // Always draw the points of a Triangle in a counter clockwise direction
  // For an explanation on this check out chapter 2 of the redbook under the heading "polygons"
  // NB: You can draw more than one Triangle between the glBegin and glEnd
  // But make sure that you have 3 points/vertices for each quad any extra do not get used
  glBegin(GL_TRIANGLES);
    glVertex2f(-10.0,-10.0);               // Set the first point of the polygon in the scene (bottom left)
    glVertex2f( 10.0,-10.0);               // Set the second point in the scene (bottom right)
    glVertex2f( 0.0, 10.0);                // Top middle

    glVertex2f(-40.0,-20.0);               // Set the first point of the polygon in the scene (bottom left)
    glVertex2f(  0.0,-20.0);               // Set the second point in the scene (bottom right)
    glVertex2f(-40.0,10.0);                // Top middle
  glEnd();                                 // End the current set of rendering instructions
end;

procedure TfrmFunc00010.DrawTriangleStrip;
begin
  // Begin by specifying what Primitive we are going to draw
  // In this case GL_TRIANGLE_STRIP...
  // A Triangle is a polygon with only three sides
  // Always draw the points of a Triangle in a counter clockwise direction
  // For an explanation on this check out chapter 2 of the redbook under the heading "polygons"
  glBegin(GL_TRIANGLE_STRIP);
    glColor3f(1.0,0.0,0.0);
    glVertex2f(-10.0,-10.0);               // Set the first point of the polygon in the scene (bottom left)
    glVertex2f( 10.0,-10.0);               // Set the second point in the scene (bottom right)
    glVertex2f( 0.0, 10.0);                // Top middle
    glColor3f(0.0,1.0,0.0);
    glVertex2f( 20.0, -5.0);               // Add next point
    glColor3f(0.0,0.0,1.0);
    glVertex2f( 20.0, 30.0);              // Add next point
  glEnd();                                 // End the current set of rendering instructions
end;

procedure TfrmFunc00010.glDraw;
begin
  glClearColor(0.0, 0.0, 0.0, 1.0); // Black Background
  glClear(GL_COLOR_BUFFER_BIT OR GL_DEPTH_BUFFER_BIT);     // Clear the colour and depth buffer
  glLoadIdentity();                 // Load a "clean" model matrix on the stack

  glColor3f(1.0,1.0,1.0);           // Set the colour of anything drawn to the screen to white 

  case CurrentDrawState of
    DRAW_POINTS: DrawPoints();
    DRAW_LINES: DrawLines();
    DRAW_LINE_STRIP: DrawLineStrip();
    DRAW_LINE_LOOP : DrawLineLoop();
    DRAW_POLYGON: DrawPolygon();
    DRAW_QUADS : DrawQuad();
    DRAW_QUAD_STRIP : DrawQuadStrip();
    DRAW_TRIANGLES: DrawTriangles();
    DRAW_TRIANGLE_STRIP: DrawTriangleStrip();
    DRAW_TRIANGLE_FAN: DrawTriangleFan();
  end;

  // Flush the OpenGL Buffer
  glFlush();                        // ( Force the buffer to draw or send a network packet of commands in a networked system)
  FContext.PageFlip;
end;

procedure TfrmFunc00010.glInit;
begin
  glClearColor(0.0, 0.0, 0.0, 1.0); // Black Background
  glColor3f(1.0, 1.0, 1.0);         // Set the Polygon colour to white

  glShadeModel(GL_FLAT);            // Use flat shading
  glEnable(GL_DEPTH_TEST);          // Enable testing the debth of any pixel on the scene.
                                    // Used for Hidden Surface Removal

  glPointSize(2.0);                 // Sets the size of all points that will be drawn in the scene
                                    // The parameter must be greater that 0.0 and its default is 1.0

  glLineWidth(4.0);                 // Set the width of all lines drawn in the scene
end;

procedure TfrmFunc00010.glResize;
begin
	with FP do
  begin
    // This is one way to set the perspective when a window get resized.
    // But this is not the only way. Another way will be handled in a later tutorial

    glViewport(0, 0, Width, Height);                    // Set the viewport for the OpenGL window
    glMatrixMode(GL_PROJECTION);                        // Change Matrix Mode to Projection
    glLoadIdentity();                                   // Reset View ( Load Thye default projection matrix onto the stack)
    if (Width <= Height) then
        glOrtho (-24.0, 24.0, -24.0*Height/Width,
            24.0*Height/Width, -1.0, 1.0)               // Set up orthographic view
    else
        glOrtho (-24.0*Width/Height,
            24.0*Width/Height, -24.0, 24.0, -1.0, 1.0); // Set up orthographic view
    glMatrixMode(GL_MODELVIEW);                         // Return to the modelview matrix
    glLoadIdentity();                                  // Reset View ( Load the default Modelview matrix onto the stack)
  end;
end;

end.

