unit fFunc00017;

interface

uses fFunc, fUtils, OpenGL, GL;

type
	TfrmFunc00017 = class(TfrmFunc)
  protected
  	procedure Subdivide(v1, v2, v3 : array of GLfloat; depth : GLint);
  public
    procedure glInit; override;
    procedure glDraw; override;
    procedure glResize; override;
  end;

const
  // User Constants

  X = 0.525731112119133606;
  Z = 0.850650808352039932;

  vdata : array [0..11] of array [0..2] of GLfloat = (
   (-X, 0.0, Z), (X, 0.0, Z), (-X, 0.0, -Z), (X, 0.0, -Z),
   (0.0, Z, X), (0.0, Z, -X), (0.0, -Z, X), (0.0, -Z, -X),
   (Z, X, 0.0), (-Z, X, 0.0), (Z, -X, 0.0), (-Z, -X, 0.0)
    );

  tindices : array [0..19] of array [0..2] of GLint = (
   (0,4,1), (0,9,4), (9,5,4), (4,5,8), (4,8,1),
   (8,10,1), (8,3,10),(5,3,8), (5,2,3), (2,7,3),
   (7,10,3), (7,6,10), (7,11,6), (11,0,6), (0,1,6),
   (6,1,10), (9,0,11), (9,11,2), (9,2,5), (7,2,11)
   );

var
  // User Variables

  spin : GLfloat = 0.0;              // Var used for the amount to rotate an object by

  DrawShaded : boolean = True;
  NumberOfDivisions : GLint = 0;


  light_ambient : array[0..3] of GLfloat = ( 0.3, 0.3, 0.3, 1.0 );
  light_diffuse : array[0..3] of GLfloat = ( 1.0, 1.0, 1.0, 1.0 );
  light_specular : array[0..3] of GLfloat = ( 1.0, 1.0, 1.0, 1.0 );
  light_position : array[0..3] of GLfloat = ( 5.0, 4.0, -4.0, 2.0 );

implementation

{ TfrmFunc00017 }

procedure TfrmFunc00017.glDraw;
var
	i: GLint;
begin
  glClearColor(0.0, 0.0, 0.0, 1.0); // Black Background
  glClear (GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);    // Clear the colour and depth buffer
  glLoadIdentity();                 // Load a "clean" model matrix on the stack

  glTranslatef(0,0,-3);      				// Move the scene back by ten units
  SpinDisplay(spin, 1);
  glRotatef(spin,0,-1,-1); 					// Rotate the Icosahedron so that we can visualise it properly
  glColor3f(1,1,1);         				// Set the colour to white
  for i := 0 to 19 do
  begin
     Subdivide(vdata[tindices[i][0]],
            vdata[tindices[i][1]],
            vdata[tindices[i][2]],NumberOfDivisions); // Subdivide and draw the Sphere
  end;

  // Flush the OpenGL Buffer
  glFlush();                        // ( Force the buffer to draw or send a network packet of commands in a networked system)
	FContext.PageFlip;
end;

procedure TfrmFunc00017.glInit;
begin
  glClearColor(0.0, 0.0, 0.0, 1.0); // Black Background
  glColor3f(1.0, 1.0, 1.0);         // Set the Polygon colour to white
  glEnable(GL_DEPTH_TEST);          // Enable Depth testing
  glDepthFunc(GL_LEQUAL);
  glShadeModel(GL_SMOOTH);            // Use smooth shading
  glHint(GL_SHADE_MODEL,GL_NICEST);   // Set the smooth shaiding to the best we can have

  // Enable Lighting ( This will be explained in a later chapter)
  glLightfv(GL_LIGHT0, GL_AMBIENT, @light_ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, @light_diffuse);
  glLightfv(GL_LIGHT0, GL_SPECULAR,@light_specular);
  glLightfv(GL_LIGHT0, GL_POSITION,@light_position);

  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);

end;

procedure TfrmFunc00017.glResize;
begin
	with FP do
  begin
    if (Height = 0) then                // prevent divide by zero exception
      Height := 1;
    glViewport(0, 0, Width, Height);    // Set the viewport for the OpenGL window
    glMatrixMode(GL_PROJECTION);        // Change Matrix Mode to Projection
    glLoadIdentity();                   // Reset View
    gluPerspective(45.0, Width/Height, 1.0, 100.0);  // Do the perspective calculations. Last value = max clipping depth

    glMatrixMode(GL_MODELVIEW);         // Return to the modelview matrix
    glLoadIdentity();                   // Reset View
  end;
end;

procedure TfrmFunc00017.Subdivide(v1, v2, v3: array of GLfloat; depth: GLint);
var
  v12 : array[0..2] of GLfloat;
  v23 : array[0..2] of GLfloat;
  v31 : array[0..2] of GLfloat;
  i : GLint; 
begin

  if depth = 0 then
  begin
    DrawTriangle(v1, v2, v3);
    exit;
  end;
  for i := 0 to 2 do
  begin
    v12[i] := v1[i]+v2[i];
    v23[i] := v2[i]+v3[i];
    v31[i] := v3[i]+v1[i];
  end;
  Normalize(v12);
  Normalize(v23);
  Normalize(v31);
  Subdivide(v1, v12, v31, depth-1);
  Subdivide(v2, v23, v12, depth-1);
  Subdivide(v3, v31, v23, depth-1);
  Subdivide(v12, v23, v31, depth-1);

end;

end.

