unit fFunc00024;

interface

uses fFunc, OpenGL, GL;

type
	TfrmFunc00024 = class(TfrmFunc)
  protected
	  procedure DrawLine();
  public
    procedure glInit; override;
    procedure glDraw; override;
    procedure glResize; override;
  end;

const
  // User Constants

  MY_LIST_NAME = 1;                  // Constant for our Display list

implementation

{ TfrmFunc00024 }

procedure TfrmFunc00024.DrawLine;
begin
  glBegin(GL_LINES);
    glVertex2f(0.0, 0.5);
    glVertex2f(15.0, 0.5);
  glEnd();
end;

procedure TfrmFunc00024.glDraw;
var
	i: GLint;
begin
  glClearColor(0.0, 0.0, 0.0, 1.0);     // Black Background
  glClear (GL_COLOR_BUFFER_BIT);        // Clear the colour buffer
  glLoadIdentity();                     // Load a "clean" model matrix on the stack
  glTranslatef(-7.0,0.0,-20.0);          // Move the scene back 10 units so we can see the arm properly

  glColor3f(0.0, 1.0, 0.0);             // Set our current rendering colour to green
                                        // This will have no effect because we set our colour to red inside the display list
                                        // See glInit()

  // Because the Display List restores its state all the triangles are drawn on top of each other
  for  i := 0 to 9 do
    glCallList(MY_LIST_NAME);           // Call The List we created in the glInit() procedure
                                        // This uses the data we added into our Video Card's memory
                                        // When we created the MY_LIST_NAME list
  DrawLine();                           // Draw one line

  // Flush the OpenGL Buffer
  glFlush();                             // ( Force the buffer to draw or send a network packet of commands in a networked system)
	FContext.PageFlip;
end;

procedure TfrmFunc00024.glInit;
begin
  glClearColor(0.0, 0.0, 0.0, 1.0); // Black Background
  glShadeModel(GL_FLAT);            // Use Flat shading

  // All the code below will only have an effect on the scene once we call the list using glCallList()
  glNewList (MY_LIST_NAME, GL_COMPILE); // Create a new Display List ( The second parameter is tellung OpenGL to compile the list but not to execute it)
    glPushMatrix();                     // Duplicate the current matrix and add it to the stack
      glPushAttrib(GL_CURRENT_BIT);     // Save the CURRENT state variables
        glColor3f(1.0, 0.0, 0.0);       // Set the colour to red
        glBegin (GL_TRIANGLES);         // Begin rendering triangles
          glVertex2f(0.0, 0.0);         // Bottom left
          glVertex2f(1.0, 0.0);         // Bottom right
          glVertex2f(0.0, 1.0);         // Top Left
        glEnd();                        // Stop drawing the primitive
        glTranslatef(1.5, 0.0, 0.0);    // Move the scene 1.5 units right
        glPopAttrib();                  // Restore the current state variables
    glPopMatrix();                      // Restore the previous matrix
  glEndList();                          // Stop creating the list

  // See example 4.2 for a Diplay List that does not restor its state after being called

  // NB: In most cases you will create all Display Lists Here,
  // since creating them in the Render Loop
  // will defeat the point of using Display Lists. (No Performance Gain)
end;

procedure TfrmFunc00024.glResize;
begin
	with FP do
  begin
    if (Height = 0) then                // prevent divide by zero exception
      Height := 1;
    glViewport(0, 0, Width, Height);    // Set the viewport for the OpenGL window
    glMatrixMode(GL_PROJECTION);        // Change Matrix Mode to Projection
    glLoadIdentity();                   // Reset View
    gluPerspective(45.0, Width/Height, 1.0, 100.0);  // Do the perspective calculations. Last value = max clipping depth

    glMatrixMode(GL_MODELVIEW);         // Return to the modelview matrix
    glLoadIdentity();                   // Reset View
  end;
end;

end.

