unit fFunc00028;

interface

uses fFunc, OpenGL, GL;

type
	TfrmFunc00028 = class(TfrmFunc)
  public
    procedure glInit; override;
    procedure glDraw; override;
    procedure glResize; override;
  end;

const
  // User Constants

  mat_specular : array [0..3] of GLfloat = ( 1.0, 1.0, 1.0, 1.0 );
  mat_shininess : GLfloat = 50.0;
  light_position : array [0..3] of GLfloat = ( 1.0, 1.0, 1.0, 0.0 );

var
  // User Variables

  // Quadric for sphere ( Dont worry too much about this now we'll handle this later)
  SphereQuadratic : gluQuadricObj;
  
implementation

{ TfrmFunc00028 }

procedure TfrmFunc00028.glDraw;
begin
  glClearColor(0.0, 0.0, 0.0, 1.0);     // Black Background
  glClear (GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);        // Clear the colour buffer
  glLoadIdentity();                     // Load a "clean" model matrix on the stack
  glTranslatef(0.0,0.0,-3.0);          // Move the scene back 10 units so we can see the triangle properly

  glColor3f(1.0,0.0,0.0);               // Set our current rendering colour to red

  gluSphere(SphereQuadratic, 1.0, 64, 64); // Render the sphere to the scene

  // Flush the OpenGL Buffer
  glFlush();                            // ( Force the buffer to draw or send a network packet of commands in a networked system)
	FContext.PageFlip;  
end;

procedure TfrmFunc00028.glInit;
begin
  glClearColor(0.0, 0.0, 0.0, 1.0); // Black Background
  glShadeModel(GL_SMOOTH);          // Use Smooth shading ( This is the Default so we dont actually have to set it) 

  // Generate a sphere using the OpenGL utility Library
  SphereQuadratic := gluNewQuadric();		       // Create A Pointer To The Quadric Object (Return 0 If No Memory) (NEW)
  gluQuadricNormals(SphereQuadratic, GLU_SMOOTH);	       // Create Smooth Normals (NEW)

  glMaterialfv(GL_FRONT, GL_SPECULAR, @mat_specular);
  glMaterialfv(GL_FRONT, GL_SHININESS, @mat_shininess);
  glLightfv(GL_LIGHT0, GL_POSITION, @light_position);

  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glDepthFunc(GL_LEQUAL);
  glEnable(GL_DEPTH_TEST);
end;

procedure TfrmFunc00028.glResize;
begin
	with FP do
  begin
    if (Height = 0) then                // prevent divide by zero exception
      Height := 1;
    glViewport(0, 0, Width, Height);    // Set the viewport for the OpenGL window
    glMatrixMode(GL_PROJECTION);        // Change Matrix Mode to Projection
    glLoadIdentity();                   // Reset View
    gluPerspective(45.0, Width/Height, 1.0, 20.0);  // Do the perspective calculations. Last value = max clipping depth

    glMatrixMode(GL_MODELVIEW);         // Return to the modelview matrix
    glLoadIdentity();                   // Reset View
  end;
end;

end.

