unit fFunc00032;

interface

uses fFunc, OpenGL, GL;

type
	TfrmFunc00032 = class(TfrmFunc)
  public
    procedure glInit; override;
    procedure glDraw; override;
    procedure glResize; override;
  end;

const
  // User Constants

  no_mat : array [0..3] of GLfloat = ( 0.0, 0.0, 0.0, 1.0 );
  mat_ambient : array [0..3] of GLfloat = ( 0.7, 0.7, 0.7, 1.0 );
  mat_ambient_color : array [0..3] of GLfloat = ( 0.8, 0.8, 0.2, 1.0 );
  mat_diffuse : array [0..3] of GLfloat = ( 0.1, 0.5, 0.8, 1.0 );
  mat_specular : array [0..3] of GLfloat = ( 1.0, 1.0, 1.0, 1.0 );
  no_shininess : GLfloat =  0.0;
  low_shininess : GLfloat =  5.0;
  high_shininess : GLfloat =  100.0;
  mat_emission : array [0..3] of GLfloat = (0.3, 0.2, 0.2, 0.0 );

var
  // User Variables

  spin : GLfloat = 0.0;              // Var used for the amount to rotate an object by

  // Quadric for sphere ( Dont worry too much about this now we'll handle this later)
  SphereQuadratic : gluQuadricObj;
  
implementation

{ TfrmFunc00032 }

procedure TfrmFunc00032.glDraw;
begin
  glClearColor(0.0, 0.0, 0.0, 1.0);     // Black Background
  glClear (GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);        // Clear the colour buffer
  glLoadIdentity();                     // Load a "clean" model matrix on the stack
  glTranslatef(0.0,0.0,-12.0);          // Move the scene back 10 units so we can see the triangle properly

  {------------------------------------------------------------------}
  { Draw sphere in first row, first column                           }
  { diffuse reflection only; no ambient or specular                  }
  {------------------------------------------------------------------}
  glPushMatrix();
    glTranslatef (-6, 0.0, 0.0);
    glMaterialfv(GL_FRONT, GL_AMBIENT, @no_mat);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, @mat_diffuse);
    glMaterialfv(GL_FRONT, GL_SPECULAR, @no_mat);
    glMaterialfv(GL_FRONT, GL_SHININESS, @no_shininess);
    glMaterialfv(GL_FRONT, GL_EMISSION, @no_mat);
    gluSphere(SphereQuadratic, 1.5, 32, 32); // Render the sphere to the scene
  glPopMatrix();

  {------------------------------------------------------------------}
  { Draw sphere in first row, second column                          }
  { diffuse and specular reflection; low shininess; no ambient       }
  {------------------------------------------------------------------}
  glPushMatrix();
    glTranslatef (-2, 0.0, 0.0);
    glMaterialfv(GL_FRONT, GL_AMBIENT, @no_mat);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, @mat_diffuse);
    glMaterialfv(GL_FRONT, GL_SPECULAR, @mat_specular);
    glMaterialfv(GL_FRONT, GL_SHININESS, @low_shininess);
    glMaterialfv(GL_FRONT, GL_EMISSION, @no_mat);
    gluSphere(SphereQuadratic, 1.5, 32, 32); // Render the sphere to the scene
  glPopMatrix();

  {------------------------------------------------------------------}
  { Draw sphere in first row, third column                           }
  { diffuse and specular reflection; high shininess; no ambient      }
  {------------------------------------------------------------------}
  glPushMatrix();
    glTranslatef (2, 0.0, 0.0);
    glMaterialfv(GL_FRONT, GL_AMBIENT, @no_mat);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, @mat_diffuse);
    glMaterialfv(GL_FRONT, GL_SPECULAR, @mat_specular);
    glMaterialfv(GL_FRONT, GL_SHININESS, @high_shininess);
    glMaterialfv(GL_FRONT, GL_EMISSION, @no_mat);
    gluSphere(SphereQuadratic, 1.5, 32, 32); // Render the sphere to the scene
  glPopMatrix();

  {------------------------------------------------------------------}
  { draw sphere in first row, fourth column                          }
  { diffuse refl.; emission; no ambient or specular reflection       }
  {------------------------------------------------------------------}
  glPushMatrix();
    glTranslatef (6, 0.0, 0.0);
    glMaterialfv(GL_FRONT, GL_AMBIENT, @no_mat);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, @mat_diffuse);
    glMaterialfv(GL_FRONT, GL_SPECULAR, @no_mat);
    glMaterialfv(GL_FRONT, GL_SHININESS, @no_shininess);
    glMaterialfv(GL_FRONT, GL_EMISSION, @mat_emission);
    gluSphere(SphereQuadratic, 1.5, 32, 32); // Render the sphere to the scene
  glPopMatrix();
 
  // Flush the OpenGL Buffer
  glFlush();                            // ( Force the buffer to draw or send a network packet of commands in a networked system)
	FContext.PageFlip;
end;

procedure TfrmFunc00032.glInit;
begin
  glClearColor(0.0, 0.0, 0.0, 1.0); // Black Background
  glShadeModel(GL_SMOOTH);          // Use Smooth shading ( This is the Default so we dont actually have to set it) 

  // Generate a sphere using the OpenGL utility Library
  SphereQuadratic := gluNewQuadric();		       // Create A Pointer To The Quadric Object (Return 0 If No Memory) (NEW)
  gluQuadricNormals(SphereQuadratic, GLU_SMOOTH);	       // Create Smooth Normals (NEW)

  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glDepthFunc(GL_LEQUAL);
  glEnable(GL_DEPTH_TEST);

end;

procedure TfrmFunc00032.glResize;
begin
	with FP do
  begin
    if (Height = 0) then                // prevent divide by zero exception
      Height := 1;
    glViewport(0, 0, Width, Height);    // Set the viewport for the OpenGL window
    glMatrixMode(GL_PROJECTION);        // Change Matrix Mode to Projection
    glLoadIdentity();                   // Reset View
    gluPerspective(45.0, Width/Height, 1.0, 20.0);  // Do the perspective calculations. Last value = max clipping depth

    glMatrixMode(GL_MODELVIEW);         // Return to the modelview matrix
    glLoadIdentity();                   // Reset View
  end;
end;

end.

