unit fFunc00035;

interface

uses fFunc, fUtils, OpenGL, GL;

type
	TfrmFunc00035 = class(TfrmFunc)
  public
    procedure glInit; override;
    procedure glDraw; override;
    procedure glResize; override;
  end;

const
  // User Constants

  // Data for the Icosahedron
  X = 0.525731112119133606;
  Z = 0.850650808352039932;

  vdata : array [0..11] of array [0..2] of GLfloat = (
   (-X, 0.0, Z), (X, 0.0, Z), (-X, 0.0, -Z), (X, 0.0, -Z),
   (0.0, Z, X), (0.0, Z, -X), (0.0, -Z, X), (0.0, -Z, -X),
   (Z, X, 0.0), (-Z, X, 0.0), (Z, -X, 0.0), (-Z, -X, 0.0)
    );

  tindices : array [0..19] of array [0..2] of GLint = (
   (0,4,1), (0,9,4), (9,5,4), (4,5,8), (4,8,1),
   (8,10,1), (8,3,10),(5,3,8), (5,2,3), (2,7,3),
   (7,10,3), (7,6,10), (7,11,6), (11,0,6), (0,1,6),
   (6,1,10), (9,0,11), (9,11,2), (9,2,5), (7,2,11)
   );

var
  // User Variables

  spin : GLfloat = 0.0;              // Var used for the amount to rotate an object by
  

implementation

{ TfrmFunc00035 }

procedure TfrmFunc00035.glDraw;
var
	i: GLint;
begin
  glClearColor(0.0, 0.0, 0.0, 1.0); // Black Background
  glClear (GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);    // Clear the colour and depth buffer
  glLoadIdentity();                 // Load a "clean" model matrix on the stack

  glColor4f(1.0, 1.0, 1.0, 1.0);    // Set the Primitive colour to white

  glTranslatef(0.0,0.0,-3.0);      // Move the scene back by 10 units
  SpinDisplay(spin,1);                    // Calculate the next Rotation Angle
  glRotatef(spin,1.0,1.0,1.0);      // Rotate the Icosahedron so that we can visualise it properly
  for i := 0 to 19 do               // Draw the Icosahedron
  begin

   glBegin(GL_TRIANGLES);
      glVertex3fv(@vdata[tindices[i][0]][0]);
      glVertex3fv(@vdata[tindices[i][1]][0]);
      glVertex3fv(@vdata[tindices[i][2]][0]);
   glEnd();
  end;
  // Flush the OpenGL Buffer
  glFlush();                        // ( Force the buffer to draw or send a network packet of commands in a networked system)
  FContext.PageFlip;
end;

procedure TfrmFunc00035.glInit;
begin
  glClearColor(0.0, 0.0, 0.0, 0.0);       // Black Background
  glColor3f(1.0, 1.0, 1.0);               // Set the Polygon colour to white

  glEnable(GL_LINE_SMOOTH);               // Use Line smooting
  glEnable(GL_BLEND);                     // Enable Blending
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); // Set the type of blending
  glHint(GL_LINE_SMOOTH_HINT, GL_NICEST); // Use the Nicest Smooting possible
  glLineWidth(1);                       // Set the line width to 1.5

  glShadeModel(GL_FLAT);                  // Do not shade the Model
  glDepthFunc(GL_LEQUAL);                 // Set the Depth Function to Linear Equal
  glEnable(GL_DEPTH_TEST);                // Enable Depth Testing

  glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);// Only draw outlines of polygons
end;

procedure TfrmFunc00035.glResize;
begin
	with FP do
  begin
    if (Height = 0) then                // prevent divide by zero exception
      Height := 1;
    glViewport(0, 0, Width, Height);    // Set the viewport for the OpenGL window
    glMatrixMode(GL_PROJECTION);        // Change Matrix Mode to Projection
    glLoadIdentity();                   // Reset View
    gluPerspective(45.0, Width/Height, 1.0, 30.0);  // Do the perspective calculations. Last value = max clipping depth

    glMatrixMode(GL_MODELVIEW);         // Return to the modelview matrix
    glLoadIdentity();                   // Reset View

  end;
end;

end.

