unit fFunc00036;

interface

uses Classes, Controls, fFunc, fUtils, OpenGL, GL;

type
	TfrmFunc00036 = class(TfrmFunc)
  protected
	  procedure CycleFog;
    procedure RenderBox(x, y, z : GLfloat);
    procedure DO_MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  public
    procedure glInit; override;
    procedure glDraw; override;
    procedure glResize; override;
  end;

const
  // User Constants

  position : array [0..3] of GLfloat = ( 0.0, 3.0, 3.0, 0.0 );
  local_view : GLfloat =  0.0 ;

var
  // User Variables

  spin : GLfloat = 0.0;              // Var used for the amount to rotate an object by

  // Fog Variables
  fogMode : GLint = GL_EXP;
  density : GLfloat;
  fogColor : array [0..3] of GLfloat = (0.4,0.4,0.4,1.0);

   // Mouse
  MouseButton : Integer = -1;         // mouse button down
  xPos, yPos, zPos : glFloat;         // Location
  Xcoord, Ycoord, Zcoord : Integer;   // Mouse Coordinates

implementation

{ TfrmFunc00036 }

procedure TfrmFunc00036.CycleFog;
begin
  case fogMode of
    GL_EXP: fogMode := GL_EXP2;
    GL_EXP2:
      begin
        fogMode := GL_LINEAR;
        glFogf(GL_FOG_START, 1.0);
        glFogf(GL_FOG_END, 5.0);
      end;
    GL_LINEAR:fogMode := GL_EXP;
  end;
  glFogi(GL_FOG_MODE, fogMode);
end;

procedure TfrmFunc00036.DO_MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
	CycleFog;
end;

procedure TfrmFunc00036.RenderBox(x, y, z: GLfloat);
var
	mat: array[0..2] of GLfloat;
begin
  glPushMatrix();
    glTranslatef (x, y, z);
    mat[0] := 0.1745; mat[1] := 0.01175; mat[2] := 0.01175;
    glMaterialfv (GL_FRONT, GL_AMBIENT, @mat);
    mat[0] := 0.61424; mat[1] := 0.04136; mat[2] := 0.04136;
    glMaterialfv (GL_FRONT, GL_DIFFUSE, @mat);
    mat[0] := 0.727811; mat[1] := 0.626959; mat[2] := 0.626959;
    glMaterialfv (GL_FRONT, GL_SPECULAR, @mat);
    glMaterialf (GL_FRONT, GL_SHININESS, 0.8*128.0);
    DrawBox(0.5,0.5,0.5);
  glPopMatrix();
end;

procedure TfrmFunc00036.glDraw;
begin
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
  glLoadIdentity();                   // Reset View
  glTranslatef(0.0,0.0,-2.0);

  RenderBox(-4.0, -0.5, -1.0);
  RenderBox(-2.0, -0.5, -2.0);
  RenderBox(0.0, -0.5, -3.0);
  RenderBox(2.0, -0.5, -4.0);
  RenderBox(4.0, -0.5, -5.0);
  // Flush the OpenGL Buffer
  glFlush();                        // ( Force the buffer to draw or send a network packet of commands in a networked system)
  FContext.PageFlip;
end;

procedure TfrmFunc00036.glInit;
begin
  FP.OnMouseDown := DO_MouseDown;
  glEnable(GL_DEPTH_TEST);
  glDepthFunc(GL_LEQUAL);
  glLightfv(GL_LIGHT0, GL_POSITION, @position);
  glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER, @local_view);

  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glEnable(GL_FOG);
  fogMode := GL_EXP;
  glFogi (GL_FOG_MODE, fogMode);
  glFogfv (GL_FOG_COLOR, @fogColor);
  glFogf (GL_FOG_DENSITY, 0.35);
  glHint (GL_FOG_HINT, GL_NICEST);
  glClearColor(0.5, 0.5, 0.5, 1.0);

end;

procedure TfrmFunc00036.glResize;
begin
	with FP do
  begin
    if (Height = 0) then                // prevent divide by zero exception
      Height := 1;
    glViewport(0, 0, Width, Height);    // Set the viewport for the OpenGL window
    glMatrixMode(GL_PROJECTION);        // Change Matrix Mode to Projection
    glLoadIdentity();                   // Reset View
    gluPerspective(45.0, Width/Height, 1.0, 10.0);  // Do the perspective calculations. Last value = max clipping depth

    glMatrixMode(GL_MODELVIEW);         // Return to the modelview matrix
    glLoadIdentity();                   // Reset View
  end;
end;

end.

