unit fFunc00041;

interface

uses fFunc, fUtils, OpenGL, GL;

type
	TfrmFunc00041 = class(TfrmFunc)
  protected
		procedure LoadStripeImage;
  public
    procedure glInit; override;
    procedure glDraw; override;
    procedure glResize; override;
  end;

const
  // User Constants

  STRIPE_IMAGE_WIDTH = 32;

  sgenparams : array [0..3] of GLfloat = (1.0, 0.0, 0.0, 0.0);

var
  // User Variables

  spin : GLfloat = 0.0;              // Var used for the amount to rotate an object by

  TextureMoveX : GLfloat = 0.0;

  TorusDL : GLuint;                  // For torus DL

   // Mouse
  MouseButton : Integer = -1;         // mouse button down
  xPos, yPos, zPos : glFloat;         // Location
  Xcoord, Ycoord, Zcoord : Integer;   // Mouse Coordinates


  stripeImage : array [0..3*STRIPE_IMAGE_WIDTH] of GLubyte;

implementation

{ TfrmFunc00041 }

procedure TfrmFunc00041.LoadStripeImage;
var
	j: GLint;
begin
    for j := 0 to STRIPE_IMAGE_WIDTH -1 do
    begin
        if j<=4 then
          stripeImage[3*j] := 255
        else
          stripeImage[3*j] := 0;
        if j>4 then
          stripeImage[3*j+1] := 255
        else
          stripeImage[3*j+1] := 0;
        stripeImage[3*j+2] := 0;
    end;
end;

procedure TfrmFunc00041.glDraw;
begin
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);// Clear the colour and depth buffer
  glLoadIdentity;                                     // Load a new matrix onto the stack

  glTranslatef(0.0,0.0,-7.0);                        // Move the scene back 10 units
  SpinDisplay(spin,1);                                // Calculate the next rotation angle
  glRotatef(spin, 1.0, 0.0, 1.0);                     // Rotate the scene on the x an z-axis
  glMatrixMode(GL_TEXTURE);                           // Switch to the Texture Matix stack
  glLoadIdentity;                                     // Load a clean matrix on the stack
  glTranslatef(TextureMoveX,0.0,0.0);                 // Move the texture on the x-axis (Remember that the texture will repeat
  glMatrixMode(GL_MODELVIEW);                         // Switch to the Modelview matrix stack
  DrawBox(1.0,1.0,1.0);                               // Draw a cube
  glCallList(TorusDL);                                // Draw a torus

  // Flush the OpenGL Buffer
  glFlush();                        // ( Force the buffer to draw or send a network packet of commands in a networked system)
  FContext.PageFlip;
  
  TextureMoveX := TextureMoveX + 0.02; // Increment the position of our texture on the x-axis

end;

procedure TfrmFunc00041.glInit;
begin
  glClearColor (0.0, 0.0, 0.0, 0.0);                            // Set the background colour to black

  LoadStripeImage();                                            // Init the 1D image
  CreateTorus(TorusDL, 0.6, 1.8, 128, 256);                             // Create a Torus
  glPixelStorei(GL_UNPACK_ALIGNMENT, 1);                        // Set the bit alignment for the stiped image
  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);  // Set the texture environment mode to modulate
  glTexParameterf(GL_TEXTURE_1D, GL_TEXTURE_WRAP_S,
      GL_REPEAT);                                               // Tile the texture on the S coordinate
  glTexParameterf(GL_TEXTURE_1D, GL_TEXTURE_MAG_FILTER,
      GL_LINEAR);                                               // Set the Magnification filter to Linear
  glTexParameterf(GL_TEXTURE_1D, GL_TEXTURE_MIN_FILTER,
      GL_LINEAR);                                               // Set the Minification filter to Linear
  glTexImage1D(GL_TEXTURE_1D, 0, 3, STRIPE_IMAGE_WIDTH, 0,
      GL_RGB, GL_UNSIGNED_BYTE, @stripeImage);                  // Load the texture into video memory
  glEnable(GL_TEXTURE_1D);                                      // Enable 1 dimentional textures

  // Here we set the texture coordinates for our environment map
  glTexGenfv(GL_S, GL_SPHERE_MAP, nil);                           // All S texture coordinates will be generated for a sphere
  glTexGenfv(GL_T, GL_SPHERE_MAP, nil);                           // All T texture coordinates will be generated for a sphere
  glEnable(GL_TEXTURE_GEN_S);                                   // Enable S texture coordinate generation
  glEnable(GL_TEXTURE_GEN_T);                                   // Enable T texture coordinate generation

  glEnable(GL_DEPTH_TEST);                                      // Enable Depth Testing
  glDepthFunc(GL_LEQUAL);                                       // Define the Depth funcion as Linear Equal

  glEnable(GL_LIGHTING);                                        // Enable Lighting
  glEnable(GL_LIGHT0);                                          // Enable Light 0

  glEnable(GL_AUTO_NORMAL);                                     // Auto generate Normals
  glEnable(GL_NORMALIZE);                                       // Enable Normalization

  glCullFace(GL_BACK);                                          // Which face do we cull
  glEnable(GL_CULL_FACE);                                       // Enable back face culling

  glMaterialf (GL_FRONT, GL_SHININESS, 64.0);                   // Set the material shininess for all object drawn in the scene
end;

procedure TfrmFunc00041.glResize;
begin
	with FP do
  begin
    if (Height = 0) then                // Prevent divide by zero exception
      Height := 1;
    glViewport(0, 0, Width, Height);    // Set the viewport for the OpenGL window
    glMatrixMode(GL_PROJECTION);        // Change Matrix Mode to Projection
    glLoadIdentity();                   // Reset View
    gluPerspective(45.0, Width/Height, 1.0, 20.0);  // Do the perspective calculations. Last value = max clipping depth

    glMatrixMode(GL_MODELVIEW);         // Return to the modelview matrix
    glLoadIdentity();                   // Reset View
  end;
end;


end.

