unit fFunc00042;

interface

uses fFunc, fUtils, OpenGL, GL;

type
	TfrmFunc00042 = class(TfrmFunc)
  public
    procedure glInit; override;
    procedure glDraw; override;
    procedure glResize; override;
  end;

const
  // User Constants
  // Displaylist constants for materials
  YELLOWMAT = 1;
  BLUEMAT = 2;

  // Lighting and materials
  yellow_diffuse : array [0..3] of GLfloat = ( 0.7, 0.7, 0.0, 1.0 );
  yellow_specular : array [0..3] of GLfloat = ( 1.0, 1.0, 1.0, 1.0 );

  blue_diffuse : array [0..3] of GLfloat = ( 0.1, 0.1, 0.7, 1.0 );
  blue_specular : array [0..3] of GLfloat = ( 0.1, 1.0, 1.0, 1.0 );

  position_one : array [0..3] of GLfloat = ( 1.0, 1.0, 1.0, 0.0 );

var
  // User Variables

  spin : GLfloat = 0.0;              // Var used for the amount to rotate an object by

  TorusDL : GLuint;                  // For torus DL
  SphereQuadric : GLUquadricObj;     // Quadric for our sphere

   // Mouse
  MouseButton : Integer = -1;         // mouse button down
  xPos, yPos, zPos : glFloat;         // Location
  Xcoord, Ycoord, Zcoord : Integer;   // Mouse Coordinates
  

implementation

{ TfrmFunc00042 }

procedure TfrmFunc00042.glDraw;
begin
   glClear(GL_STENCIL_BUFFER_BIT);                           // Clear the stencil buffer

    //create a diamond shaped stencil area
    glMatrixMode(GL_PROJECTION);                             // Use the Projection Matrix
    glPushMatrix();                                          // Copy the current matrix and push it onto the projection matrix stack
      glLoadIdentity();                                      // Load a new Projection Matrix onto the stack 
      glOrtho(-3.0, 3.0, -3.0, 3.0, -1.0, 1.0);              // Setup the orthographic view
      glMatrixMode(GL_MODELVIEW);                            // Switch to the Modelview matrix
      glPushMatrix();                                        // Copy the current Modelview matrix and add it to the stack
        glLoadIdentity();                                    // Load a new matrix onto the stack

        // Disable color buffer update.
        glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE); // Set the masks used to control writing into the stencil buffer to false
        glDisable(GL_DEPTH_TEST);                            // Disable Depth testing
        glStencilFunc (GL_ALWAYS, 1, 1);                     // Always set the First stencil buffers pixel value to one where we render
        glStencilOp (GL_REPLACE, GL_REPLACE, GL_REPLACE);    // Replace any old data in the Stencil buffer

        glBegin(GL_QUADS);                                   // Draw a quad into the stencil buffer
          glVertex3f (-1.0, 0.0, 0.0);
          glVertex3f (0.0, 1.0, 0.0);
          glVertex3f (1.0, 0.0, 0.0);
          glVertex3f (0.0, -1.0, 0.0);
        glEnd();

      glPopMatrix();                                          // Restore the previous Modelview matrix
      glMatrixMode(GL_PROJECTION);                            // Switch to Projection Matrix
    glPopMatrix();                                            // Restore the old projection matrix
    glMatrixMode(GL_MODELVIEW);                               // Switch to Model View Matrix

    // Enable color buffer update.
    glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);          // Set the masks used to control writing into the stencil buffer to True
    glEnable(GL_DEPTH_TEST);                                  // Enable Depth Testing
    glStencilOp (GL_KEEP, GL_KEEP, GL_KEEP);                  // Keep everyting we draw now

    glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT );     // Clear the Colour and Depth Buffer


    glStencilFunc(GL_EQUAL, 1, 1);                            // draw blue sphere where the stencil is 1
    glCallList (BLUEMAT);                                     // Set the material to Blue

    gluSphere(SphereQuadric,0.5,32,32);                       // Draw a Sphere


    glStencilFunc(GL_NOTEQUAL, 1, 1);                         // draw the tori where the stencil is not 1
    glPushMatrix();                                           // Save the current Modelview Matrix
      glRotatef (45.0, 0.0, 0.0, 1.0);                        // Rotate 45 degrees around the z-axis
      glRotatef (45.0, 0.0, 1.0, 0.0);                        // Rotate 45 degrees arounf the y-axis
      glCallList (YELLOWMAT);                                 // Set the material to Yellow
      glCallList(TorusDL);                                    // Draw a Torus
      glPushMatrix();                                         // Save the current Mpdelview matrix
          glRotatef (90.0, 1.0, 0.0, 0.0);                    // Rotate 90 degrees on the x-axis
          glCallList(TorusDL);                                // Draw another Torus
      glPopMatrix();                                          // Restore the previous Modelview matrix
    glPopMatrix();                                            // Restore the previous Modelview matrix

    glFlush();              // ( Force the buffer to draw or send a network packet of commands in a networked system)
    FContext.PageFlip;
end;

procedure TfrmFunc00042.glInit;
begin
  glClearColor (0.0, 0.0, 0.0, 0.0);                          // Set the background colour to black
  SphereQuadric := gluNewQuadric();		                        // Create A Pointer To The Quadric Object (Return 0 If No Memory) (NEW)
  gluQuadricNormals(SphereQuadric, GLU_SMOOTH);	              // Create Smooth Normals (NEW)

  glNewList(YELLOWMAT, GL_COMPILE);                           // Compile a new Displaylist
    glMaterialfv(GL_FRONT, GL_DIFFUSE, @yellow_diffuse);      // Set the Material Diffuse Colour
    glMaterialfv(GL_FRONT, GL_SPECULAR, @yellow_specular);    // Set the Material Specular Colour
    glMaterialf(GL_FRONT, GL_SHININESS, 64.0);                // Set the Material Shininess
  glEndList();                                                // End the list

  glNewList(BLUEMAT, GL_COMPILE);                             // Compile a new Displaylist
    glMaterialfv(GL_FRONT, GL_DIFFUSE, @blue_diffuse);        // Set the Material Diffuse Colour
    glMaterialfv(GL_FRONT, GL_SPECULAR, @blue_specular);      // Set the Material Specular Colour
    glMaterialf(GL_FRONT, GL_SHININESS, 45.0);                // Set the Material Shininess
  glEndList();                                                // End the list

  CreateTorus(TorusDL, 0.275, 0.85, 128, 256);                        // Create a Torus

  glLightfv(GL_LIGHT0, GL_POSITION, @position_one);           // Set the light position

  glEnable(GL_LIGHT0);                                        // Enable light 0
  glEnable(GL_LIGHTING);                                      // Enable lighting
  glDepthFunc(GL_LEQUAL);                                     // Set the Depth function to Linear Equal
  glEnable(GL_DEPTH_TEST);                                    // Enable Depth testing

  glClearStencil(0);                                          // Set the clearing value of the stencil to 0 (This is the default so this call is actually unnecesary)
  glEnable(GL_STENCIL_TEST);                                  // Enable the stencil buffer testing
  glTranslatef(0.0, 0.0, -5.0);                               // Moce the scene back 5 units
end;

procedure TfrmFunc00042.glResize;
begin
	with FP do
  begin
    if (Height = 0) then                // Prevent divide by zero exception
      Height := 1;
     glViewport(0, 0, Width, Height);   // Set the Viewport to be as big as the window size

      glMatrixMode(GL_PROJECTION);      // Switch to the Projection Matrix
      glLoadIdentity();                 // Load a new Projection Matrix
      gluPerspective(45.0, Width/ Height, 3.0, 7.0); // Set the Perspective for the scene
      glMatrixMode(GL_MODELVIEW);       // Switch to the Modelview Matrix
  end;

end;

end.

