unit fFunc00044;

interface

uses Math, fFunc, fUtils, OpenGL, GL;

type
	TJitterPoint = record
		x, y : GLfloat;
  end;

	TfrmFunc00044 = class(TfrmFunc)
  protected
		procedure accFrustum(dleft, dright, dbottom, dtop, dnear, dfar,
    	dpixdx, dpixdy, deyedx, deyedy, dfocus : GLdouble );
    procedure accPerspective(dfovy, daspect, dnear, dfar, dpixdx,
    	dpixdy, deyedx, deyedy, dfocus : GLdouble);
    procedure CreateIcosahedron();
    procedure DrawObjects();
  public
    procedure glInit; override;
    procedure glDraw; override;
    procedure glResize; override;
  end;

const
  // User Constants

  // Icosahedron constants
  X = 0.525731112119133606;
  Z = 0.850650808352039932;

  vdata : array [0..11] of array [0..2] of GLfloat = (
   (-X, 0.0, Z), (X, 0.0, Z), (-X, 0.0, -Z), (X, 0.0, -Z),
   (0.0, Z, X), (0.0, Z, -X), (0.0, -Z, X), (0.0, -Z, -X),
   (Z, X, 0.0), (-Z, X, 0.0), (Z, -X, 0.0), (-Z, -X, 0.0)
    );

  tindices : array [0..19] of array [0..2] of GLint = (
   (0,4,1), (0,9,4), (9,5,4), (4,5,8), (4,8,1),
   (8,10,1), (8,3,10),(5,3,8), (5,2,3), (2,7,3),
   (7,10,3), (7,6,10), (7,11,6), (11,0,6), (0,1,6),
   (6,1,10), (9,0,11), (9,11,2), (9,2,5), (7,2,11)
   );


  // Lighting and materials
  mat_ambient : array [0..3] of GLfloat = ( 1.0, 1.0, 1.0, 1.0 );
  mat_specular : array [0..3] of GLfloat = ( 1.0, 1.0, 1.0, 1.0 );
  light_position : array [0..3] of GLfloat = (0.0, 0.0, 10.0, 1.0 );
  lm_ambient : array [0..3] of GLfloat = ( 0.2, 0.2, 0.2, 1.0 );

  torus_diffuse : array [0..3] of GLfloat = ( 0.7, 0.7, 0.0, 1.0 );
  cube_diffuse : array [0..3] of GLfloat = ( 0.0, 0.7, 0.7, 1.0 );
  sphere_diffuse : array [0..3] of GLfloat = (0.7, 0.0, 0.7, 1.0 );
  octa_diffuse : array [0..3] of GLfloat = ( 0.7, 0.4, 0.4, 1.0 );

  // Jitter constants
  ACSIZE = 8;

var
  // User Variables

  spin : GLfloat = 0.0;              // Var used for the amount to rotate an object by

  TorusDL : GLuint;                  // For torus DL
  CubeDL : GLuint;                   // For Cube DL
  IcosDL : GLuint;                   // For Icosahedron
  SphereQuadric : GLUquadricObj;     // Quadric for our sphere

   // Mouse
  MouseButton : Integer = -1;         // mouse button down
  Xcoord, Ycoord : Integer;   // Mouse Coordinates
  
  
implementation

{ TfrmFunc00044 }

procedure TfrmFunc00044.accFrustum(dleft, dright, dbottom, dtop, dnear,
  dfar, dpixdx, dpixdy, deyedx, deyedy, dfocus: GLdouble);
var
  xwsize, ywsize :GLdouble;
  dx, dy : GLdouble;
  viewport : array [0..3] of GLint;
begin

    glGetIntegerv(GL_VIEWPORT, @viewport);                         // Get the current viewport

    xwsize := dright - dleft;
    ywsize := dtop - dbottom;
    dx := -(dpixdx * xwsize/viewport[2] + deyedx * dnear/dfocus);
    dy := -(dpixdy * ywsize/viewport[3] + deyedy * dnear/dfocus);

    glMatrixMode(GL_PROJECTION);                                   // Switch the the Projection Matrix
    glLoadIdentity();                                              // Load A new matix onto the stack 
    glFrustum (dleft + dx, dright + dx, dbottom + dy, dtop + dy, dnear, dfar);  // Set the frustrum
    glMatrixMode(GL_MODELVIEW);                                    // Switch to the Modelview matrix
    glLoadIdentity();                                              // Load A new matix onto the stack
    glTranslatef (-deyedx, -deyedy, 0.0);                          // Translate the scene according to the eye coordinates 
end;

procedure TfrmFunc00044.accPerspective(dfovy, daspect, dnear, dfar, dpixdx,
  dpixdy, deyedx, deyedy, dfocus: GLdouble);
var
  fov2, left, right, bottom, top : GLdouble;
begin

    fov2 := ((dfovy*PI) / 180.0) / 2.0;

    top := dnear / (ArcCos(fov2) / ArcSin(fov2));
    bottom := -top;
    right := top * daspect;
    left := -right;

    accFrustum (left, right, bottom, top, dnear, dfar,
        dpixdx, dpixdy, deyedx, deyedy, dfocus);        // Setup the accumilation frustrum
end;

procedure TfrmFunc00044.CreateIcosahedron;
var
	i :GLint;
begin
  IcosDL :=glGenLists(1);
  glNewList(IcosDL, GL_COMPILE);
    for i := 0 to 19 do
    begin
        glBegin(GL_TRIANGLES);
          glNormal3fv(@vdata[tindices[i][0]][0]);
          glVertex3fv(@vdata[tindices[i][0]][0]);
          glNormal3fv(@vdata[tindices[i][1]][0]);
          glVertex3fv(@vdata[tindices[i][1]][0]);
          glNormal3fv(@vdata[tindices[i][2]][0]);
          glVertex3fv(@vdata[tindices[i][2]][0]);
      glEnd();
    end;
  glEndList;
end;

procedure TfrmFunc00044.DrawObjects;
begin
  glPushMatrix ();                                           // Load a new matrix onto the stack
    glTranslatef (0.0, 0.0, -5.0);                           // Maove the scene back 5 units
    glRotatef (30.0, 1.0, 0.0, 0.0);                         // Rotate the scene 30 degrees on the x-axis

    glPushMatrix ();                                         // Load a new matrix onto the stack
      glTranslatef (-0.80, 0.35, 0.0);                       // Move the scene
      glRotatef (100.0, 1.0, 0.0, 0.0);                      // Rotate 100 degrees on the x-axis
      glMaterialfv(GL_FRONT, GL_DIFFUSE, @torus_diffuse);    // Set the Torus material
      glCallList(TorusDL);                                   // Draw the Torus
    glPopMatrix ();                                          // Restore the last saved matrix

   glPushMatrix ();                                          // Load a new matrix onto the stack
      glTranslatef (-0.75, -0.50, 0.0);                      // Move the scene
      glRotatef (45.0, 0.0, 0.0, 1.0);                       // Rotate 45 degrees on the z-axis
      glRotatef (45.0, 1.0, 0.0, 0.0);                       // Rotate 45 degrees on the x-axis
      glMaterialfv(GL_FRONT, GL_DIFFUSE, @cube_diffuse);     // Set the Cube material
      glCallList(CubeDL);                                    // Draw the cube
    glPopMatrix ();                                          // Restore the last saved matrix

    glPushMatrix ();                                         // Load a new matrix onto the stack
      glTranslatef (0.75, 0.60, 0.0);                        // Move the scene
      glRotatef (30.0, 1.0, 0.0, 0.0);                       // Rotate 30 degrees on the x-axis
      glMaterialfv(GL_FRONT, GL_DIFFUSE, @sphere_diffuse);   // Set the Sphere material
      gluSphere(SphereQuadric,1.0,32,32);                    // Draw the sphere
    glPopMatrix ();                                          // Restore the last saved matrix

    glPushMatrix ();                                         // Load a new matrix onto the stack
      glTranslatef (0.70, -0.90, 0.25);                      // Move the scene
      glMaterialfv(GL_FRONT, GL_DIFFUSE, @octa_diffuse);     // Set the Icosahedron material
      glCallList(IcosDL);                                    // Draw the Icosahedron
    glPopMatrix ();                                          // Restore the last saved matrix

  glPopMatrix ();                                            // Restore the matrix we saved initially
end;

procedure TfrmFunc00044.glDraw;
var
  viewport : array [0..3] of GLint;
  jitter : GLint;
begin


    glGetIntegerv (GL_VIEWPORT, @viewport);                      // Get the vurrent View port
    glClear(GL_ACCUM_BUFFER_BIT);                                // Clear the Accumulation Buffer

    for jitter := 0 to ACSIZE -1 do
    begin
      glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);       // Clear the Colour and Depth Buffer
      glPushMatrix();                                            // Copy the current Marix and add it to the stack
       {	Note that 4.5 is the distance in world space between
       *	left and right and bottom and top.
       *	This formula converts fractional pixel movement to
       *	world coordinates.
       }
        glTranslatef (j8[jitter].x*4.5/viewport[2],
            j8[jitter].y*4.5/viewport[3], 0.0);                  // Move the scene
        DrawObjects ();                                          // Draw the Objects
      glPopMatrix ();                                            // Restore the last save Matrix
      glAccum(GL_ACCUM, 1.0/ACSIZE);                             // Accumulate the Scene
    end;
    glAccum (GL_RETURN, 1.0);                                    // Render the acumulated scene


    glFlush();              // ( Force the buffer to draw or send a network packet of commands in a networked system)
		FContext.PageFlip;
end;

procedure TfrmFunc00044.glInit;
begin
  glMaterialfv(GL_FRONT, GL_AMBIENT, @mat_ambient);     // Set the Ambient Material
  glMaterialfv(GL_FRONT, GL_SPECULAR, @mat_specular);   // Set the Specular Material
  glMaterialf(GL_FRONT, GL_SHININESS, 50.0);            // Set the Material Shininess
  glLightfv(GL_LIGHT0, GL_POSITION, @light_position);   // Set the Light Position
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, @lm_ambient);  // Set the Light Ambient Value

  glEnable(GL_LIGHTING);                                // Enable Lighting
  glEnable(GL_LIGHT0);                                  // Enable Light 0
  glDepthFunc(GL_LEQUAL);                               // Set the Depth Funtion To Linear Equal
  glEnable(GL_DEPTH_TEST);                              // Enable Depth Testing (Hidden Surface Removal)
  glShadeModel (GL_FLAT);                               // Use Flat Shading
  glClearColor(0.0, 0.0, 0.0, 0.0);                     // Initialize the Colour buffer to black
  glClearAccum(0.0, 0.0, 0.0, 0.0);                     // Initialize the Accumulation buffer to black

  CreateTorus(TorusDL, 0.275, 0.85, 10, 20);                    // Create a Torus

  CubeDL := glGenLists(1);                              // Create a Cube
  glNewList(CubeDL, GL_COMPILE);
    DrawBox(1.5,1.5,1.5);
  glEndList;

  SphereQuadric := gluNewQuadric();		                 // Create A Pointer To The Quadric Object (Return 0 If No Memory) (NEW)
  gluQuadricNormals(SphereQuadric, GLU_SMOOTH);	       // Create Smooth Normals (NEW)

  CreateIcosahedron();                                 // Create an Icosahedron
end;

procedure TfrmFunc00044.glResize;
begin
	with FP do
  begin
    glViewport(0, 0, Width, Height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    if (Width <= Height) then
      glOrtho (-2.25, 2.25, -2.25*Width/Height, 2.25*Height/Width, -10.0, 10.0)
    else
      glOrtho (-2.25*Width/Height, 2.25*Width/Height, -2.25, 2.25, -10.0, 10.0);
    glMatrixMode(GL_MODELVIEW);
  end;
end;

end.

