unit fFunc00047;

interface

uses fFunc, fUtils, OpenGL, GL, GLext;

type
	TfrmFunc00047 = class(TfrmFunc)
  protected
  	procedure InitLights();
  public
    procedure glInit; override;
    procedure glDraw; override;
    procedure glResize; override;
  end;

const
  // User Constants

  // Lighting

  ambient : array [0..3] of GLfloat = ( 0.2, 0.2, 0.2, 1.0 );
  position : array [0..3] of GLfloat = ( 0.0, 0.0, 2.0, 1.0  );
  mat_diffuse : array [0..3] of GLfloat = ( 0.6, 0.6, 0.6, 1.0 );
  mat_specular : array [0..3] of GLfloat = ( 1.0, 1.0, 1.0, 1.0 );
  mat_shininess : GLfloat =  50.0 ;

var
  // User Variables

  spin : GLfloat = 0.0;              // Var used for the amount to rotate an object by

   // Mouse
  MouseButton : Integer = -1;         // mouse button down
  Xcoord, Ycoord : Integer;   // Mouse Coordinates

  // Control Points for Bezier Surface
  ctrlpoints : array [0..3] of array [0..3] of array[0..2] of GLfloat = (
    ((-1.5, -1.5, 4.0), (-0.5, -1.5, 2.0),
     (0.5, -1.5, -1.0), (1.5, -1.5, 2.0)),
    ((-1.5, -0.5, 1.0), (-0.5, -0.5, 3.0),
      (0.5, -0.5, 0.0), (1.5, -0.5, -1.0)),
    ((-1.5, 0.5, 4.0), (-0.5, 0.5, 0.0),
      (0.5, 0.5, 3.0), (1.5, 0.5, 4.0)),
    ((-1.5, 1.5, -2.0), (-0.5, 1.5, -2.0),
      (0.5, 1.5, 0.0), (1.5, 1.5, -1.0))
  );

implementation

{ TfrmFunc00047 }

procedure TfrmFunc00047.InitLights;
begin
  glEnable(GL_LIGHTING);                                       // Enable Light 0
  glEnable(GL_LIGHT0);
  glLightfv(GL_LIGHT0, GL_AMBIENT, @ambient);                  // The the Ambient value for the light
  glLightfv(GL_LIGHT0, GL_POSITION, @position);                // Set the Position of the light
  glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, @mat_diffuse);   // Set the Diffuse Material
  glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, @mat_specular); // Set the Specular Material
  glMaterialfv(GL_FRONT_AND_BACK,GL_SHININESS, @mat_shininess);// Set the Shininess of the Material
end;

procedure TfrmFunc00047.glDraw;
begin
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);         // Clear the Colour and Depth Buffer
  glPushMatrix();                                              // Copy the current matrix and add it to the stack
      glRotatef(85.0, 1.0, 1.0, 1.0);                          // Rotate the scene
      glEvalMesh2(GL_FILL, 0, 8, 0, 8);                        // Draw A Filled Bezier Surface
  glPopMatrix();                                               // Revert back to the saved matrix
  glFlush();              // ( Force the buffer to draw or send a network packet of commands in a networked system)
  FContext.PageFlip;
end;

procedure TfrmFunc00047.glInit;
begin
  glClearColor (0.0, 0.0, 0.0, 1.0);                          // Set the background colour to black
  glEnable(GL_DEPTH_TEST);                                    // Enable Depth testing
  glMap2f(GL_MAP2_VERTEX_3, 0, 1, 3, 4,                       // Set the control points for the bezier surface
      0, 1, 12, 4, @ctrlpoints[0][0][0]);
  glEnable(GL_MAP2_VERTEX_3);                                 // Enable Vertex gereration
  glEnable(GL_AUTO_NORMAL);                                   // Generate Normals Automatically
  glMapGrid2f(8, 0.0, 1.0, 8, 0.0, 1.0);                      // Set the grid for the Bezier surface
  InitLights();                                               // Setup Lights and Materials
end;

procedure TfrmFunc00047.glResize;
begin
	with FP do
  begin
    glViewport(0, 0, Width, Height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    if (Width <= Height) then
        glOrtho(-3.15, 3.15, -3.15*Height/Width,3.15*Height/Width, -3.15, 3.15)
    else
        glOrtho(-3.15*Width/Height, 3.15*Width/Height, -3.15, 3.15, -3.15, 3.15);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
  end;
end;


end.

