unit fFunc00048;

interface

uses fFunc, OpenGL, GL;

type
	TfrmFunc00048 = class(TfrmFunc)
  protected
	  procedure LoadImage();
  public
    procedure glInit; override;
    procedure glDraw; override;
    procedure glResize; override;
  end;

const
  // User Constants

  // Image Constants
  IMAGE_WIDTH = 64;
  IMAGE_HEIGHT = 64;

var
  // User Variables

  spin : GLfloat = 0.0;              // Var used for the amount to rotate an object by

  SphereQuadric : GLUquadricObj;     // Quadric for our sphere

   // Mouse
  MouseButton : Integer = -1;         // mouse button down
  Xcoord, Ycoord : Integer;   // Mouse Coordinates

  // Control Point for Bezier Surface
  ctrlpoints : array [0..3] of array [0..3] of array[0..2] of GLfloat = (
    ((-1.5, -1.5, 4.0), (-0.5, -1.5, 2.0),
     (0.5, -1.5, -1.0), (1.5, -1.5, 2.0)),
    ((-1.5, -0.5, 1.0), (-0.5, -0.5, 3.0),
      (0.5, -0.5, 0.0), (1.5, -0.5, -1.0)),
    ((-1.5, 0.5, 4.0), (-0.5, 0.5, 0.0),
      (0.5, 0.5, 3.0), (1.5, 0.5, 4.0)),
    ((-1.5, 1.5, -2.0), (-0.5, 1.5, -2.0),
      (0.5, 1.5, 0.0), (1.5, 1.5, -1.0))
  );

  // Control Points for texture
  texpts : array [0..1] of array [0..1] of array[0..1] of GLfloat = (
    ((0.0, 0.0), (0.0, 1.0)),
    ((1.0, 0.0), (1.0, 1.0))
    );

  // Texture Data
  image : array [0..3*IMAGE_WIDTH*IMAGE_HEIGHT] of GLubyte;

implementation

{ TfrmFunc00048 }

procedure TfrmFunc00048.glDraw;
begin
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT); // Clear the colour and depth buffer
  glLoadIdentity();                                    // Load a new Matrix onto the stack
  glTranslatef(-2,-1,0);
  glRotatef(85.0, 1.0, 1.0, 1.0);                      // Rotate the scene
  glPushMatrix();                                      // Copy the current matrix and push it onto the stack
      glRotatef(85.0, 1.0, 1.0, 1.0);                  // Rotate the scene
      glEvalMesh2(GL_LINE, 0, 8, 0, 8);								 // Fill the Mesh//GL_LINE,GL_FILL
  glPopMatrix();                                       // Resture the last saved matrix
  glFlush();              // ( Force the buffer to draw or send a network packet of commands in a networked system)
  FContext.PageFlip;
end;

procedure TfrmFunc00048.glInit;
begin
  glMap2f(GL_MAP2_VERTEX_3, 0, 1, 3, 4,
      0, 1, 12, 4, @ctrlpoints[0][0][0]);                    // Add the control points for the Mesh
  glMap2f(GL_MAP2_TEXTURE_COORD_2, 0, 1, 2, 2,
      0, 1, 4, 2, @texpts[0][0][0]);                         // Add the Texture COntrol Points for the mesh
  glEnable(GL_MAP2_TEXTURE_COORD_2);                         // Enable Auto Texture U V Generation
  glEnable(GL_MAP2_VERTEX_3);                                // Enable Auto Vertex Generation
  glMapGrid2f(20, 0.0, 1.0, 20, 0.0, 1.0);                   // Create a 20x20 grid
  LoadImage();                                               // Load the Image
  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);  // Set the Texturing to Decal
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,
                   GL_REPEAT);                               // Repeat on the Texture on S
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,
                   GL_REPEAT);                               // Repeat on the Texture on T
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER,
                   GL_NEAREST);                              // Set the Magnification factor to use the nearest pixels
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
                   GL_NEAREST);                              // Set the Minification factor to use the nearest pixels
  glTexImage2D(GL_TEXTURE_2D, 0, 3, IMAGE_WIDTH, IMAGE_HEIGHT,
                   0, GL_RGB, GL_UNSIGNED_BYTE, @image);     // Load the texture into Video Memory
  glEnable(GL_TEXTURE_2D);                                   // Enable Texuring
  glEnable(GL_DEPTH_TEST);                                   // Enable Depth testing
  glEnable(GL_NORMALIZE);                                    // Auto Normalize
  glShadeModel (GL_FLAT);                                    // Do Not shade the model
end;

procedure TfrmFunc00048.glResize;
begin
	with FP do
  begin
    glViewport(0, 0, Width, Height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    if (Width <= Height) then glOrtho(-4.0, 4.0, -4.0*Height/Width, 4.0*Height/Width, -4.0, 4.0)
    else                      glOrtho(-4.0*Width/Height, 4.0*Width/Height, -4.0, 4.0, -4.0, 4.0);
    glMatrixMode(GL_MODELVIEW);
  end;
end;

procedure TfrmFunc00048.LoadImage;
var
	i, j : GLint;
  ti, tj : single;
begin
    for i := 0 to IMAGE_WIDTH -1 do
    begin
        ti := 2.0*3.141592654*i/IMAGE_WIDTH;
        for j := 0 to IMAGE_HEIGHT -1 do
        begin
            tj := 2.0*3.14159265*j/IMAGE_HEIGHT;
            image[3*(IMAGE_HEIGHT*i+j)] := Round(127*(1.0+sin(ti)));
            image[3*(IMAGE_HEIGHT*i+j)+1] := Round(127*(1.0+cos(2*tj)));
            image[3*(IMAGE_HEIGHT*i+j)+2] := Round(127*(1.0+cos(ti+tj)));
        end;
    end;
end;

end.

