unit fFunc00049;

interface

uses fFunc, OpenGL, GL;

type
	TfrmFunc00049 = class(TfrmFunc)
  protected
	  procedure InitSurface();
  public
    procedure glInit; override;
    procedure glDraw; override;
    procedure glResize; override;
  end;

const
  // User Constants

  // Image Constants
  IMAGE_WIDTH = 64;
  IMAGE_HEIGHT = 64;


  // Lighting and material

  mat_diffuse : array [0..3] of GLfloat = ( 0.7, 0.7, 0.7, 1.0 );
  mat_specular : array [0..3] of GLfloat = ( 1.0, 1.0, 1.0, 1.0 );
  mat_shininess : GLfloat = 100.0 ;

var
  // User Variables

  spin : GLfloat = 0.0;              // Var used for the amount to rotate an object by

  SphereQuadric : GLUquadricObj;     // Quadric for our sphere

   // Mouse
  MouseButton : Integer = -1;         // mouse button down
  Xcoord, Ycoord : Integer;   // Mouse Coordinates

  // Control Points for Nurbs Surface
  ctlpoints : array [0..3] of array [0..3] of array[0..2] of GLfloat;

  // Nurbs
  theNurb : GLUnurbsObj;
  knots : array [0..7] of GLfloat= (0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0);

implementation

{ TfrmFunc00049 }

procedure TfrmFunc00049.glDraw;
begin
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

  glPushMatrix();
      glRotatef(330.0, 1.0,0.0,0.0);
      glScalef (0.5, 0.5, 0.5);

      gluBeginSurface(theNurb);
      gluNurbsSurface(theNurb, 
          8, @knots,
          8, @knots,
          4 * 3,
          3,
          @ctlpoints[0][0][0],
          4, 4,
          GL_MAP2_VERTEX_3);
      gluEndSurface(theNurb);

  glPopMatrix();

  glFlush();              // ( Force the buffer to draw or send a network packet of commands in a networked system)
  FContext.PageFlip;


end;

procedure TfrmFunc00049.glInit;
begin

  glClearColor (0.0, 0.0, 0.0, 1.0);
  glMaterialfv(GL_FRONT, GL_DIFFUSE, @mat_diffuse);
  glMaterialfv(GL_FRONT, GL_SPECULAR, @mat_specular);
  glMaterialfv(GL_FRONT, GL_SHININESS, @mat_shininess);

  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glDepthFunc(GL_LEQUAL);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_AUTO_NORMAL);
  glEnable(GL_NORMALIZE);
  InitSurface();

  theNurb := gluNewNurbsRenderer();
  gluNurbsProperty(theNurb, GLU_SAMPLING_TOLERANCE, 25.0);
  gluNurbsProperty(theNurb, GLU_DISPLAY_MODE, GLU_FILL);

end;

procedure TfrmFunc00049.glResize;
begin
	with FP do
  begin
    glViewport(0, 0, Width, Height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective (45.0, Width/Height, 3.0, 8.0);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glTranslatef (0.0, 1, -5.0);
  end;
end;

procedure TfrmFunc00049.InitSurface;
var
	u, v : GLint;
begin
    for u := 0 to 3 do
    begin
        for v := 0 to 3 do
        begin
            ctlpoints[u][v][0] := 2.0*(u - 1.5);
            ctlpoints[u][v][1] := 2.0*(v - 1.5);

          if ( ((u = 1) or (u = 2)) and ((v = 1) or (v = 2))) then
              ctlpoints[u][v][2] := 3.0
          else
              ctlpoints[u][v][2] := -3.0;
       end;
    end;
end;

end.

