unit fFunc00063;

interface

uses Windows, SysUtils, fFuncNeHe, OpenGL, GL, GLaux;

type
  particles = record
    active: bool;
    life: GLfloat;
    fade: GLfloat;
    r, g, b: GLfloat;
    x, y, z: GLfloat;
    xi, yi, zi: GLfloat;
    xg, yg, zg: GLfloat;
  end;

	TfrmFunc00063 = class(TfrmFuncNeHe)
  protected
	  function LoadGLTextures: Bool;
  public
    procedure glInit; override;
    procedure glDraw; override;
  end;

procedure glGenTextures(n: GLsizei; var textures: GLuint); stdcall; external opengl32;
procedure glBindTexture(target: GLenum; texture: GLuint); stdcall; external opengl32;

const
  MAX_PARTICLES = 1000; 

var
  rainbow: bool = true;
  sp: bool;
  rp: bool;
  slowdown: GLfloat = 2.0;
  xspeed, yspeed: GLfloat;
  zoom: GLfloat = -40.0;
  loop: gluint;
  col: gluint;
  delay: gluint;
  texture: array [0..0] of gluint;
  particle: array [0..MAX_PARTICLES-1] of particles;
  colors: array [0..11,0..2] of glfloat = ((1.0,0.5,0.5),(1.0,0.75,0.5),(1.0,1.0,0.5),(0.75,1.0,0.5),
                                           (0.5,1.0,0.5),(0.5,1.0,0.75),(0.5,1.0,1.0),(0.5,0.75,1.0),
                                           (0.5,0.5,1.0),(0.75,0.5,1.0),(1.0,0.5,1.0),(1.0,0.5,0.75));
  
implementation

{ TfrmFunc00063 }

procedure TfrmFunc00063.glDraw;
var
	I: Integer;
	x, y, z: GLfloat;
begin
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
  glLoadIdentity();
  for I:=0 to MAX_PARTICLES-1 do
  begin
    if particle[I].active then
    begin
      x := particle[I].x;
      y := particle[I].y;
      z := particle[I].z+zoom;

      glColor4f(particle[I].r,particle[I].g,particle[I].b,particle[I].life);
      glBegin(GL_TRIANGLE_STRIP);
        glTexCoord2d(1,1); glVertex3f(x+0.5,y+0.5,z);
        glTexCoord2d(0,1); glVertex3f(x-0.5,y+0.5,z);
        glTexCoord2d(1,0); glVertex3f(x+0.5,y-0.5,z);
        glTexCoord2d(0,0); glVertex3f(x-0.5,y-0.5,z);
      glEnd;
      particle[I].x := particle[I].x + particle[I].xi/(slowdown*1000);
      particle[I].y := particle[I].y + particle[I].yi/(slowdown*1000);
      particle[I].z := particle[I].z + particle[I].zi/(slowdown*1000);
      particle[I].xi := particle[I].xi + particle[I].xg;
      particle[I].yi := particle[I].yi + particle[I].yg;
      particle[I].zi := particle[I].zi + particle[I].zg;
      particle[I].life := particle[I].life - particle[I].fade;
      if (particle[I].life < 0.0) then
      begin
        particle[I].life := 1.0;
        particle[I].fade := (random(100)/1000.0) + 0.003;
        particle[I].x := 0.0;
        particle[I].y := 0.0;
        particle[I].z := 0.0;
        particle[I].xi := xspeed + (random(60) - 32.0);
        particle[I].yi := yspeed + (random(60) - 30.0);
        particle[I].zi := (random(60) - 30.0);
        particle[I].r := colors[col, 0];
        particle[I].g := colors[col, 1];
        particle[I].b := colors[col, 2];
      end;
  	end;
  end;

(*
      if (keys[VK_NUMPAD8] AND (particle[I].yg < 1.5)) then particle[I].yg := particle[I].yg + 0.01;

      if (keys[VK_NUMPAD2] AND (particle[I].yg > -1.5)) then particle[I].yg := particle[I].yg - 0.01;

      if (keys[VK_NUMPAD6] AND (particle[I].xg < 1.5)) then particle[I].xg := particle[I].xg + 0.01;

      if (keys[VK_NUMPAD4] AND (particle[I].xg > -1.5)) then particle[I].xg := particle[I].xg - 0.01;
      if (keys[VK_TAB]) then
      begin
        particle[I].x := 0.0;
        particle[I].y := 0.0;
        particle[I].z := 0.0;
        particle[I].xi := (random(50)-26.0)*10.0;
        particle[I].yi := (random(50)-25.0)*10.0;
        particle[I].zi := (random(50)-25.0)*10.0;
      end;
      end;
    end;
*)
	FContext.PageFlip;    
end;

procedure TfrmFunc00063.glInit;
var
	I: Integer;
begin
  if not LoadGLTextures then EXIT;
  glShadeModel(GL_SMOOTH);
  glClearColor(0.0, 0.0, 0.0, 0.0);
  glClearDepth(1.0);
  glDisable(GL_DEPTH_TEST);
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA,GL_ONE);
  glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
  glHint(GL_POINT_SMOOTH_HINT,GL_NICEST);
  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D,texture[0]);
  for I:=0 to MAX_PARTICLES-1 do
  begin
    particle[I].active := true;
    particle[I].life := 1.0;
    particle[I].fade := Random(100)/1000 + 0.003;
    particle[I].r := colors[Trunc(I*(12/MAX_PARTICLES)),0];
    particle[I].g := colors[Trunc(I*(12/MAX_PARTICLES)),1];
    particle[I].b := colors[Trunc(I*(12/MAX_PARTICLES)),2];
    particle[I].xi := (random(50)-26.0)*10.0;
    particle[I].yi := (random(50)-25.0)*10.0;
    particle[I].zi := (random(50)-25.0)*10.0;
    particle[I].xg := 0.0;
    particle[I].yg := -0.8;
    particle[I].zg := 0.0;
  end;
end;

function TfrmFunc00063.LoadGLTextures: Bool;
var
	TextureImage: array [0..0] of PTAUX_RGBImageRec;
  Status: Bool;
begin
  Status := False;
  ZeroMemory(@TextureImage,sizeof(TextureImage));
  TextureImage[0] := LoadBMP('C:\Particle.bmp');
  if Assigned(TextureImage[0]) then
  begin
    Status := true;
    glGenTextures(1,Texture[0]);
    glBindTexture(GL_TEXTURE_2D,texture[0]);
    glTexImage2D(GL_TEXTURE_2D,0,3,TextureImage[0].sizeX,
    	TextureImage[0].sizeY,0,GL_RGB,GL_UNSIGNED_BYTE,TextureImage[0].data);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
  end;
  Result := Status;
end;

end.

