unit fMain;

interface

uses
  Windows, Messages, Graphics, Forms, Classes, Menus, fFunc, Math, SysUtils;

const
  MAX_FUNC_COUNT = 63;

	WM_DOT 						= WM_USER + 4000;
  WM_NEWACTIVEFUNC 	= WM_DOT  + 1;		

type
	TfrmMain = class;

	TCadencerThread = class(TThread)
  protected
  	FOwner: TfrmMain;
  	procedure Execute; override;
    procedure DO_INVALIDATE;
  public
  	constructor Create(AOwner: TfrmMain);
  end;

  TfrmMain = class(TForm)
    MMenu: TMainMenu;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
  	CadencerThread: TCadencerThread;
  	miFuncs: array[0..9] of TMenuItem;
  	miItems: array[0..9, 0..9] of TMenuItem;
  	FPrevFuncList: TStringList;
    FActiveFuncType: Integer;
    FActiveFunc: TfrmFunc;
    procedure CREATE_MENUITEM;
    procedure DO_MENUITEM_CLICK(Sender: TObject);
    procedure SetActiveFuncType(const Value: Integer);
    function  GetPreviousFuncType: Integer;
    procedure SetPreviousFuncType(const Value: Integer);
    procedure WMNewActiveFunc(var Message: TMessage); message WM_NEWACTIVEFUNC;
	protected
		procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    procedure Resize; override;
  public
  	property ActiveFuncType: Integer read FActiveFuncType write SetActiveFuncType;
    property PreviousFuncType: Integer read GetPreviousFuncType write SetPreviousFuncType;
  end;

var
  frmMain: TfrmMain;
	IsAddToPrevFuncList: Boolean;

implementation

uses fUtils;

{$R *.dfm}

{ TCadencerThread }

constructor TCadencerThread.Create(AOwner: TfrmMain);
begin
	inherited Create(True);
  FOwner := AOwner;
  FreeOnTerminate := True;
  Resume;
end;

procedure TCadencerThread.DO_INVALIDATE;
begin
	TfrmFunc(FOwner.FActiveFunc).OnPaint(FOwner);
end;

procedure TCadencerThread.Execute;
begin
	while not Terminated do
  begin
  	if Assigned(FOwner.FActiveFunc) then
    	Synchronize(DO_INVALIDATE);
    Application.ProcessMessages;
    Sleep(10);
  end;
	Terminate;
end;

{ TfrmMain }

procedure TfrmMain.CREATE_MENUITEM;
var
	I, Count: Integer;
  C, R: Integer;
begin
	MMenu.Items.Clear;
	for I := 0 to Length(miFuncs)-1 do
  begin
  	miFuncs[I] := TMenuItem.Create(Self);
    miFuncs[I].Tag := I;
    miFuncs[I].Name := 'miFunc_' + IntToStr(I);
 		miFuncs[I].Caption := 'Func_' + IntToStr(I);
    miFuncs[I].Enabled := (I <= (MAX_FUNC_COUNT-1) div 10);
  end;
 	MMenu.Items.Add(miFuncs);

  Count := Length(miItems) * Length(miItems[0]);
  for I := 0 to Count-1 do
  begin
  	C := I div 10;
    R := I mod 10;
    miItems[C,R] := TMenuItem.Create(Self);
    miItems[C,R].Tag 		:= I;
    miItems[C,R].Name 	:= 'miFuncItem_' + IntToStr(I);
 		miItems[C,R].Caption := 'Func_' + Format('%5.5d',[I]);
    miItems[C,R].OnClick := DO_MENUITEM_CLICK;
    miItems[C,R].Enabled := I < MAX_FUNC_COUNT;
  end;

  for I := 0 to Length(miItems)-1 do
  	miFuncs[I].Add(miItems[I]);

end;

procedure TfrmMain.DO_MENUITEM_CLICK(Sender: TObject);
var
	I: Integer;
begin
	I := TComponent(Sender).Tag;
  if I < MAX_FUNC_COUNT then
  begin
		I := I mod MAX_FUNC_COUNT;
		ActiveFuncType := I;
  end;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
	IsAddToPrevFuncList := True;
	FActiveFuncType := -1;
	CREATE_MENUITEM;
  FPrevFuncList := TStringList.Create;
	KeyPreview := True;
	Color := clBlack;
	WindowState := wsMaximized;
	ActiveFuncType := 0;//MAX_FUNC_COUNT-1;//Working Function
  CadencerThread := TCadencerThread.Create(Self);
end;

procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
	if CadencerThread <> nil then
  	CadencerThread.Terminate;
  CadencerThread := nil;
	FreeAndNil(FPrevFuncList);
end;

function TfrmMain.GetPreviousFuncType: Integer;
begin
	Result := -1;
	if FPrevFuncList.Count > 0 then
		Result := StrToInt(FPrevFuncList[0]);
end;

procedure TfrmMain.SetPreviousFuncType(const Value: Integer);
begin
	if not IsAddToPrevFuncList then EXIT;
  if (Value>-1) and (Value<>PreviousFuncType) then
  	FPrevFuncList.Insert(0, IntToStr(Value));
end;

procedure TfrmMain.WMNewActiveFunc(var Message: TMessage);
begin
	ActiveFuncType := Message.wParam;
end;

procedure TfrmMain.SetActiveFuncType(const Value: Integer);
var
	pf, ff: TfrmFunc;//Previous Function Form
begin
	if Value <> FActiveFuncType then
  begin
  	PreviousFuncType := FActiveFuncType;
	  FActiveFuncType := Max(0, Value);
    pf := FActiveFunc;
    FActiveFunc := CreateFunc(Self,FActiveFuncType);
    if Assigned(FActiveFunc) then Resize;
    if Assigned(pf) then
    begin
    	ff := TfrmFunc(FindComponent(pf.Name));
      if Assigned(ff) then RemoveComponent(ff);
    	pf.Close;
    end;
  end;
end;

procedure TfrmMain.KeyDown(var Key: Word; Shift: TShiftState);
begin
	case Key of
  	VK_ESCAPE:
    	if PreviousFuncType > -1 then
    	begin
      	IsAddToPrevFuncList := False;
        ActiveFuncType := PreviousFuncType;
        with FPrevFuncList do if Count > 0 then Delete(0);
        IsAddToPrevFuncList := True;
      end;
    VK_F1: ActiveFuncType := (ActiveFuncType + MAX_FUNC_COUNT + 1) mod MAX_FUNC_COUNT;
    else
    	if Assigned(FActiveFunc) and Assigned(FActiveFunc.OnKeyDown) then
      	FActiveFunc.OnKeyDown(Self, Key, Shift);
  end;
  Key := 0;
end;

procedure TfrmMain.Resize;
begin
	if Assigned(FActiveFunc) then FActiveFunc.OnResize(Self);
end;

procedure TfrmMain.KeyPress(var Key: Char);
begin
	if Assigned(FActiveFunc) and Assigned(FActiveFunc.OnKeyPress) then
  	FActiveFunc.OnKeyPress(Self, Key);
end;

end.
